-- column size from TYPE NAME,TYPE MODE,ATTRIBUTE LEN
create or replace function pg_jdbc_column_size(text,integer,integer) returns integer as '
declare
	vtypname alias for $1;
	vtypmode alias for $2;
	vatlen alias for $3;
	result integer:=null;
begin
	if vtypname = ''varchar'' then
		if vtypmode = -1 then
			result:=null;
		else
			result:=vtypmode-4;
		end if;
	elsif vtypname = ''bpchar'' then
		if vtypmode = -1 then
			result:=null;
		else
			result:=vtypmode-4;
		end if;
	elsif vtypname = ''numeric'' then
		if vtypmode = -1 then
			result:=null;
		else
			result:=(vtypmode-4) >> 16;
		end if;
	elsif vtypname = ''decimal'' then
		if vtypmode = -1 then
			result:=null;
		else
			result:=(vtypmode-4) >> 16;
		end if;
	else
		result:=vatlen;
	end if;
	return(result);
end;
' language 'plpgsql';
-- precision for numeric and decimal type for TYPE NAME and TYPMOD
create or replace function pg_jdbc_precision(text,integer) returns integer as '
declare
	vtypname alias for $1;
	vtypmode alias for $2;
	result integer:=null;
begin
	if vtypname = ''numeric'' then
		if vtypmode = -1 then
			result:=null;
		else
			result:=(int4(vtypmode)-int4(''4'')) & int4(''65535'');
		end if;
	end if;
	if vtypname = ''decimal'' then
		if vtypmode = -1 then
			result:=null;
		else
			result:=(int4(vtypmode)-int4(''4'')) & int4(''65535'');
		end if;
	end if;
	return(result);
end;
' language 'plpgsql';
-- if oid in oidvector return -1 if oidn not in oidvector else number of parameter
create or replace function pg_jdbc_oidinoidvector(oid,oidvector,integer) returns integer as '
declare
	vid alias for $1;
	vidv alias for $2;
	args alias for $3;
begin
  for i in 0..args loop
	if vid = vidv[i] then
		return (i+1);
	end if;
  end loop;
  return -1;
end;
' language 'plpgsql';
-- convert postgresql type in java (incomplete)
create or replace function pg_jdbc_postgresinjavatypes(name) returns integer as '
declare
	vname alias for $1;
	result integer;
begin
	if vname = ''int2'' then
		result:=5;
	elsif vname = ''int4'' then
		result:=4;
	elsif vname = ''oid'' then
		result:=4;
	elsif vname = ''int8'' then
		result:=-5;
	elsif vname = ''cash'' then
		result:=8;
	elsif vname = ''money'' then
		result:=8;
	elsif vname = ''numeric'' then
		result:=2;
	elsif vname = ''float4'' then
		result:=7;
	elsif vname = ''float8'' then
		result:=8;
	elsif vname = ''bpchar'' then
		result:=1;
	elsif vname = ''char'' then
		result:=1;
	elsif vname = ''char2'' then
		result:=1;
	elsif vname = ''char4'' then
		result:=1;
	elsif vname = ''char8'' then
		result:=1;
	elsif vname = ''char16'' then
		result:=1;
	elsif vname = ''varchar'' then
		result:=12;
	elsif vname = ''text'' then
		result:=12;
	elsif vname = ''name'' then
		result:=12;
	elsif vname = ''filename'' then
		result:=12;
	elsif vname = ''bytea'' then
		result:=-2;
	elsif vname = ''bool'' then
		result:=-7;
	elsif vname = ''date'' then
		result:=91;
	elsif vname = ''time'' then
		result:=92;
	elsif vname = ''abstime'' then
		result:=93;
	elsif vname = ''timestamp'' then
		result:=93;
	elsif vname = ''timestampz'' then
		result:=93;
	else 
		result:=1111;
	end if;
	return result;
end;
' language 'plpgsql';
-- view for DatabaseMetData.getColumns
create or replace view pg_jdbc_getcolumns as
select null::varchar as "TABLE CAT",
	n.nspname::varchar as "TABLE_SCHEM",
	c.relname as "TABLE_NAME",a.attname as "COLUMN_NAME",
	pg_jdbc_postgresinjavatypes(t.typname) as "DATA_TYPE",
	typname as "TYPE_NAME",
	pg_jdbc_column_size(typname,atttypmod,attlen)::integer as "COLUMN_SIZE",
	null::int4 as "BUFFER_LENGTH",
	pg_jdbc_column_size(typname,atttypmod,attlen)::integer as "DECIMAL_DIGITS",
	pg_jdbc_precision(typname,atttypmod)::integer as "NUM_PREC_RADIX"
from pg_catalog.pg_namespace n,pg_catalog.pg_class c,pg_catalog.pg_type t,pg_catalog.pg_attribute a
where c.oid=a.attrelid and t.oid=a.atttypid
  and n.oid=c.relnamespace
  and pg_catalog.pg_table_is_visible(c.oid);
-- view for DatabaseMetaData.getProcedureColumns
create or replace view pg_jdbc_getprocedurecolumns as
select 
	null::varchar as "PROCEDURE_CAT",
	n.nspname as "PROCEDURE_SCHEM",
	p.proname as "PROCEDURE_NAME",
	tr.typname as "COLUMN_NAME",
	4::int4 as "COLUMN_TYPE", -- IN parametar
	pg_jdbc_postgresinjavatypes(tr.typname) as "DATA_TYPE", 
	tr.typname as "TYPE_NAME",
	pg_jdbc_precision(tr.typname,typtypmod) as "PRECISION",
	pg_jdbc_column_size(tr.typname,typtypmod,null) as "LENGTH",
	null::integer as "SCALE",
	null::integer as "RADIX",
	null::integer as "NULLABLE",
	null::varchar as "REMARKS",
	0::integer as "ARG_ROWNUM"
from pg_catalog.pg_namespace n,pg_catalog.pg_type tr,pg_catalog.pg_proc p
where n.oid=p.pronamespace
and tr.oid=p.prorettype
union
select 
	null::varchar as "PROCEDURE_CAT",
	n.nspname as "PROCEDURE_SCHEM",
	p.proname as "PROCEDURE_NAME",
	tr.typname as "COLUMN_NAME",
	1::int4 as "COLUMN_TYPE", -- IN parametar
	pg_jdbc_postgresinjavatypes(tr.typname) as "DATA_TYPE", 
	tr.typname as "TYPE_NAME",
	pg_jdbc_precision(tr.typname,typtypmod) as "PRECISION",
	pg_jdbc_column_size(tr.typname,typtypmod,null) as "LENGTH",
	null::integer as "SCALE",
	null::integer as "RADIX",
	null::integer as "NULLABLE",
	null::varchar as "REMARKS",
	pg_jdbc_oidinoidvector(tr.oid,p.proargtypes,p.pronargs) as "ARG_ROWNUM"
from pg_catalog.pg_namespace n,pg_catalog.pg_type tr,pg_catalog.pg_proc p
where n.oid=p.pronamespace
and pg_jdbc_oidinoidvector(tr.oid,p.proargtypes,p.pronargs) > 0;
